#!/usr/bin/python

"""
Robust estimation of a line
"""

import numpy

class LineEstimation(object):
  """
  Allows the computation of a line equation in a robust way (German McClure)
  """
  def __init__(self, x, y, sigma):
    """
    The equation is defined by x and y
    the parameter sigma is the 
    """
    self.absisses = x
    self.ordinates = y
    self.sigma = sigma

  def __call__(self, x):
    """
    Computes the cost for the equation and the parameter x
    """
    D = (self.ordinates - self.absisses * x[1] - x[0])
    return D * D / (self.sigma * self.sigma + numpy.dot(D, D))

  def gradient(self, x):
    """
    Computes the gradient
    """
    D = (self.ordinates - self.absisses * x[1] - x[0])
    denominator = (self.sigma * self.sigma + numpy.dot(D, D))
    return -2. * self.sigma * self.sigma * numpy.array((numpy.dot(self.absisses, D),  numpy.sqrt(numpy.dot(D, D))), numpy.float)  / (denominator * denominator)


if __name__ == '__main__':
  import criterion
  import step
  import optimizer

  import numpy.random

  x = numpy.random.random_sample((100,))
  y = 30. * x + 2. + numpy.random.standard_exponential((100,))

  startPoint = numpy.array((3,20), numpy.float)

  print "Standard optimization with a gradient step, monotony criterion"
  optimi = optimizer.StandardOptimizer(function = LineEstimation(x, y, 10.), stepKind = step.GradientStep(), criterion = criterion.MonotonyCriterion(0.0000001, 1000), x0 = startPoint, lineSearch = line_search.SimpleLineSearch())
  print optimi.optimize()
  print "Standard optimization with a gradient step, relative value criterion"
  optimi = optimizer.StandardOptimizer(function = LineEstimation(x, y, 10.), stepKind = step.GradientStep(), criterion = criterion.RelativeValueCriterion(0.0000001, 1000), x0 = startPoint, lineSearch = line_search.SimpleLineSearch())
  print optimi.optimize()
  print "Standard optimization with a gradient step, absolute value criterion"
  optimi = optimizer.StandardOptimizer(function = LineEstimation(x, y, 10.), stepKind = step.GradientStep(), criterion = criterion.AbsoluteValueCriterion(0.0000001, 1000), x0 = startPoint, lineSearch = line_search.SimpleLineSearch())
  print optimi.optimize()
