#!/usr/bin/python

"""
Class defining the Powell function
"""

import numpy

class Powell:
  """
  The Powell function
  """
  def __call__(self, x):
    """
    Get the value of the quadratic function at a specific point
    """
    return (x[0] + 10* x[1])**2 + 5 * (x[2] - x[3])**2 + (x[1] - 2 * x[2])**4 + 10 * (x[0] - x[3])**4

  def gradient(self, x):
    """
    Evaluates the gradient of the function
    """
    return numpy.array([2 * (x[0] + 10 * x[1]) + 40 * (x[0] - x[3])**3, 20 * (x[0] + 10 * x[1]) + 4 * (x[1] - x[2])**3, 10 * (x[2] - x[3]) - 8 * (x[1] - 2 * x[2])**3, 10 * (x[2] - x[3]) - 40 * (x[0] - x[3])**3], dtype = numpy.float)

  def hessian(self, x):
    """
    Evaluates the gradient of the function
    """
    return numpy.array([[2 + 120 * (x[0] - x[3]) ** 2, 20, 0, -120 * (x[0] - x[3]) ** 2], [20, 200 + 12 * (x[1] - 2 * x[2]) ** 2, -24 * (x[1] - 2 * x[2]) ** 2, 0], [0, -24 * (x[1] - 2 * x[2]) ** 2, 10 + 48 * (x[1] - 2 * x[2]) ** 2, -10], [-120 * (x[0] - x[3]) ** 2, 0, -10, -10 + 120 * (x[0] - x[3])**2]], dtype = numpy.float)

if __name__ == '__main__':
  import criterion
  import step
  import optimizer
  import line_search

  startPoint = numpy.empty(4, numpy.float)
  startPoint[0] = 3.
  startPoint[1] = -1.
  startPoint[2] = 0.
  startPoint[3] = 1.

  print "Standard optimization with a gradient step, relative value criterion"
  optimi = optimizer.StandardOptimizer(function = Powell(), step_kind = step.GradientStep(), criterion = criterion.RelativeValueCriterion(0.000001, 10000), x0 = startPoint, line_search = line_search.SimpleLineSearch(stepSize = 0.00001))
  print optimi.optimize()

  print "Standard optimization with a Newton step, relative value criterion"
  optimi = optimizer.StandardOptimizer(function = Powell(), step_kind = step.NewtonStep(), criterion = criterion.RelativeValueCriterion(0.000001, 1000), x0 = startPoint, line_search = line_search.SimpleLineSearch())
  print optimi.optimize()

  print "Standard optimization with a gradient step, relative value criterion, golden section line search"
  optimi = optimizer.StandardOptimizer(function = Powell(), step_kind = step.GradientStep(), criterion = criterion.RelativeValueCriterion(0.000001, 10000), x0 = startPoint, line_search = line_search.GoldenSectionSearch(stepSize = 0.001, minStepSize = 0.000001))
  print optimi.optimize()

  print "Standard optimization with a FR conjugate gradient step, relative value criterion, golden section line search"
  optimi = optimizer.StandardOptimizer(function = Powell(), step_kind = step.FRConjugateGradientStep(), criterion = criterion.RelativeValueCriterion(0.000001, 1000), x0 = startPoint, line_search = line_search.GoldenSectionSearch(stepSize = 0.001, minStepSize = 0.000001))
  print optimi.optimize()

  print "Standard optimization with a Newton step, relative value criterion, golden section line search"
  optimi = optimizer.StandardOptimizer(function = Powell(), step_kind = step.NewtonStep(), criterion = criterion.RelativeValueCriterion(0.000001, 10000), x0 = startPoint, line_search = line_search.GoldenSectionSearch(minStepSize = 0.00001))
  print optimi.optimize()

  print "Standard optimization with a Newton step, relative value criterion, Fibonacci section line search"
  optimi = optimizer.StandardOptimizer(function = Powell(), step_kind = step.NewtonStep(), criterion = criterion.RelativeValueCriterion(0.000001, 10000), x0 = startPoint, line_search = line_search.FibonacciSectionSearch(minStepSize = 0.0001))
  print optimi.optimize()
