#!/usr/bin/python

"""
Class defining a quadratic function
"""

import numpy

class Quadratic:
  """
  A simple quadratic function
  """
  def __call__(self, x):
    """
    Get the value of the quadratic function at a specific point
    """
    return (x[0] + 2* x[1] - 7)**2 + (2 * x[0] + x[1] - 5)**2

  def gradient(self, x):
    """
    Evaluates the gradient of the function
    """
    return numpy.array([2 * (x[0] + 2* x[1] - 7) + 4 * (2 * x[0] + x[1] - 5), 4 * (x[0] + 2* x[1] - 7) + 2 * (2 * x[0] + x[1] - 5)], dtype = numpy.float)

  def hessian(self, x):
    """
    Evaluates the gradient of the function
    """
    return numpy.array([[10, 8], [8, 10]], dtype = numpy.float)

if __name__ == '__main__':
  import criterion
  import step
  import optimizer
  import line_search

  startPoint = numpy.empty(2, numpy.float)
  startPoint[0] = 0.
  startPoint[-1] = 0.

  print "Standard optimization with a gradient step, relative value criterion"
  optimi = optimizer.StandardOptimizer(function = Quadratic(), step_kind = step.GradientStep(), criterion = criterion.RelativeValueCriterion(0.00001, 10000), x0 = startPoint, line_search = line_search.SimpleLineSearch(stepSize = 0.001))
  print optimi.optimize()

  print "Standard optimization with a Newton step, relative value criterion"
  optimi = optimizer.StandardOptimizer(function = Quadratic(), step_kind = step.NewtonStep(), criterion = criterion.RelativeValueCriterion(0.00001, 100), x0 = startPoint, line_search = line_search.SimpleLineSearch())
  print optimi.optimize()

  print "Damped optimization with a gradient step, relative value criterion"
  optimi = optimizer.StandardOptimizer(function = Quadratic(), step_kind = step.GradientStep(), criterion = criterion.RelativeValueCriterion(0.00001, 10000), x0 = startPoint, line_search = line_search.DampedLineSearch(dampedError = 0.001, minStepSize = 0.0001))
  print optimi.optimize()

  print "Damped optimization with a gradient step, absolute value criterion"
  optimi = optimizer.StandardOptimizer(function = Quadratic(), step_kind = step.GradientStep(), criterion = criterion.AbsoluteValueCriterion(0.00001, 10000), x0 = startPoint, line_search = line_search.DampedLineSearch(dampedError = 0.001, minStepSize = 0.0001))
  print optimi.optimize()

  print "Standard optimization with a FR conjugate gradient step, relative value criterion, Fibonacci section line search"
  optimi = optimizer.StandardOptimizer(function = Quadratic(), step_kind = step.FRConjugateGradientStep(), criterion = criterion.RelativeValueCriterion(0.000001, 10000), x0 = startPoint, line_search = line_search.FibonacciSectionSearch(stepSize = .001, minStepSize = 0.0000001))
  print optimi.optimize()

  print "Standard optimization with a FR conjugate gradient step, relative value criterion, golden section line search"
  optimi = optimizer.StandardOptimizer(function = Quadratic(), step_kind = step.FRConjugateGradientStep(), criterion = criterion.RelativeValueCriterion(0.000001, 10000), x0 = startPoint, line_search = line_search.GoldenSectionSearch(stepSize = .001, minStepSize = 0.0000001))
  print optimi.optimize()
