
"""
Line Search with the Fibonacci section method
"""

import math

class FibonacciSectionSearch(object):
  """
  Line Search with the Fibonacci section method, optimal section method
  """
  def __init__(self, **kwargs):
    """
    Needs to have :
      - a minimum step size (minStepSize)
    Can have :
      - a step modifier, a factor to modulate the step (stepSize = 1.)
    """
    self.minStepSize = kwargs.get('minStepSize', 1.)
    self.stepSize = kwargs.get('stepSize', 1.)

    limit = self.stepSize / self.minStepSize
    self.fibonacci = self.computeFibonacci(limit)

  def __call__(self, origin, step, function, state, **kwargs):
    """
    Returns a good candidate
    Parameters :
      - origin is the origin of the search
      - step is the step that will be used
      - function is the function to minimize
      - state is the state of the optimizer
    """
    ak = 0
    v_ak = function(origin + ak * step)
    bk = self.stepSize
    v_bk = function(origin + bk * step)

    k = 1

    uk = ak + (self.fibonacci[-2] / self.fibonacci[-1]) * (bk - ak)
    v_uk = function(origin + uk * step)
    lk = ak + self.fibonacci[-3] / self.fibonacci[-1] * (bk - ak)
    v_lk = function(origin + lk * step)

    while True:
      if v_uk < v_lk:
        if k > len(self.fibonacci) - 2:
          state['step_size'] = uk
          return origin + uk * step
        ak = lk
        v_ak = v_lk
        lk = uk
        v_lk = v_uk
        uk = ak + self.fibonacci[-k - 1] / self.fibonacci[-k] * (bk - ak)
        v_uk = function(origin + uk * step)
      else:
        if k > len(self.fibonacci) - 2:
          state['step_size'] = lk
          return origin + lk * step
        bk = uk
        v_bk = v_uk
        uk = lk
        v_uk = v_lk
        lk = ak + self.fibonacci[-k - 2] / self.fibonacci[-k] * (bk - ak)
        v_lk = function(origin + lk * step)
      k = k + 1
        
  def computeFibonacci(self, limit):
    fibonacci = [1., 1.]
    while (fibonacci[-1] < limit):
      fibonacci.append(fibonacci[-2] + fibonacci[-1])

    return fibonacci