
"""
Computes the Fletcher-Reeves conjugate gradient step for a specific function at a specific point
"""

import numpy

class FRConjugateGradientStep(object):
  """
  The simple gradient step
  Parameters :
  - gradientSize is the size f the gradient, needed for initialization of the step
  """
  def __init__(self):
    """
    Initialization of the gradient step
    """
    self.oldStep = None
    self.oldGradient = None
  def __call__(self, function, point, state):
    """
    Computes a gradient step based on a function and a point
    """
    newGradient = function.gradient(point)

    if 'step' in state:
      coeff = numpy.dot(newGradient.T, newGradient) / numpy.dot(state['gradient'].T, newGradient)
      step = - newGradient + coeff * state['step']
    else:
      coeff = 0
      step = - newGradient
    self.oldGradient = newGradient
    state['gradient'] = newGradient
    state['step_modifier'] = coeff
    state['step'] = step
    return step

  def update(self, **kwargs):
    """
    Updates the step when needed
    """
    pass
