
"""
Computes a Newton step for a specific function at a specific point
"""

import numpy
import numpy.linalg

class NewtonStep(object):
  """
  The simple gradient step
  """
  def __call__(self, function, point, state):
    """
    Computes a gradient step based on a function and a point
    """
    hessian = function.hessian(point)
    gradient = function.gradient(point)
    step = (-numpy.dot(numpy.linalg.inv(hessian), gradient[:,numpy.newaxis])).reshape(point.shape)
    state['hessian'] = hessian
    state['gradient'] = gradient
    state['step'] = step
    return step

  def update(self, **kwargs):
    """
    Updates the step when needed
    """
    pass
